@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-
 from . import _version
 
 __version__ = _version.get_versions()["version"]
@@ -27,13 +26,9 @@ except:
 def _install():
     from xoscar.backends.router import Router
 
-    from .model import _install as install_model
-
     default_router = Router.get_instance_or_empty()
     Router.set_instance(default_router)
 
-    install_model()
-
 
 _install()
 del _install
