@@ -1,9 +1,6 @@
 from .restful.restful_client import (  # noqa: F401
     RESTfulAudioModelHandle as AudioModelHandle,
 )
-from .restful.restful_client import (  # noqa: F401
-    RESTfulChatglmCppChatModelHandle as ChatglmCppChatModelHandle,
-)
 from .restful.restful_client import (  # noqa: F401
     RESTfulChatModelHandle as ChatModelHandle,
 )
@@ -16,3 +13,6 @@ from .restful.restful_client import (  # noqa: F401
 from .restful.restful_client import (  # noqa: F401
     RESTfulImageModelHandle as ImageModelHandle,
 )
+from .restful.restful_client import (  # noqa: F401
+    RESTfulVideoModelHandle as VideoModelHandle,
+)
