@@ -13,22 +13,23 @@
 # limitations under the License.
 
 import base64
+import html
 import logging
 import os
+import tempfile
 from io import BytesIO
 from typing import Generator, List, Optional
 
 import gradio as gr
 import PIL.Image
+from gradio import ChatMessage
 from gradio.components import Markdown, Textbox
 from gradio.layouts import Accordion, Column, Row
 
 from ..client.restful.restful_client import (
-    RESTfulChatglmCppChatModelHandle,
     RESTfulChatModelHandle,
     RESTfulGenerateModelHandle,
 )
-from ..types import ChatCompletionMessage
 
 logger = logging.getLogger(__name__)
 
@@ -66,17 +67,21 @@ class GradioInterface:
 
     def build(self) -> "gr.Blocks":
         if "vision" in self.model_ability:
-            interface = self.build_chat_vl_interface()
+            interface = self.build_chat_multimodel_interface()
         elif "chat" in self.model_ability:
             interface = self.build_chat_interface()
         else:
             interface = self.build_generate_interface()
 
-        interface.queue()
+        interface.queue(default_concurrency_limit=os.cpu_count())
         # Gradio initiates the queue during a startup event, but since the app has already been
         # started, that event will not run, so manually invoke the startup events.
         # See: https://github.com/gradio-app/gradio/issues/5228
-        interface.startup_events()
+        try:
+            interface.run_startup_events()
+        except AttributeError:
+            # compatibility
+            interface.startup_events()
         favicon_path = os.path.join(
             os.path.dirname(os.path.abspath(__file__)),
             os.path.pardir,
@@ -88,74 +93,119 @@ class GradioInterface:
         interface.favicon_path = favicon_path
         return interface
 
-    def build_chat_interface(
-        self,
-    ) -> "gr.Blocks":
-        def flatten(matrix: List[List[str]]) -> List[str]:
-            flat_list = []
-            for row in matrix:
-                flat_list += row
-            return flat_list
-
-        def to_chat(lst: List[str]) -> List[ChatCompletionMessage]:
-            res = []
-            for i in range(len(lst)):
-                role = "assistant" if i % 2 == 1 else "user"
-                res.append(ChatCompletionMessage(role=role, content=lst[i]))
-            return res
-
+    def build_chat_interface(self) -> "gr.Blocks":
         def generate_wrapper(
             message: str,
-            history: List[List[str]],
+            history: List[ChatMessage],
             max_tokens: int,
             temperature: float,
             lora_name: str,
+            stream: bool,
         ) -> Generator:
             from ..client import RESTfulClient
 
             client = RESTfulClient(self.endpoint)
             client._set_token(self._access_token)
             model = client.get_model(self.model_uid)
-            assert isinstance(
-                model, (RESTfulChatModelHandle, RESTfulChatglmCppChatModelHandle)
-            )
+            assert isinstance(model, RESTfulChatModelHandle)
 
-            response_content = ""
-            for chunk in model.chat(
-                prompt=message,
-                chat_history=to_chat(flatten(history)),
-                generate_config={
-                    "max_tokens": int(max_tokens),
-                    "temperature": temperature,
-                    "stream": True,
-                    "lora_name": lora_name,
-                },
-            ):
-                assert isinstance(chunk, dict)
-                delta = chunk["choices"][0]["delta"]
-                if "content" not in delta:
+            # Convert history to messages format
+            messages = []
+            for msg in history:
+                # ignore thinking content
+                if msg["metadata"]:
                     continue
-                else:
-                    response_content += delta["content"]
-                    yield response_content
+                messages.append({"role": msg["role"], "content": msg["content"]})
 
-            yield response_content
+            if stream:
+                response_content = ""
+                reasoning_content = ""
+                is_first_reasoning_content = True
+                is_first_content = True
+                for chunk in model.chat(
+                    messages=messages,
+                    generate_config={
+                        "max_tokens": int(max_tokens),
+                        "temperature": temperature,
+                        "stream": True,
+                        "lora_name": lora_name,
+                    },
+                ):
+                    assert isinstance(chunk, dict)
+                    delta = chunk["choices"][0]["delta"]
 
-        return gr.ChatInterface(
-            fn=generate_wrapper,
-            additional_inputs=[
-                gr.Slider(
-                    minimum=1,
-                    maximum=self.context_length,
-                    value=512,
-                    step=1,
-                    label="Max Tokens",
-                ),
-                gr.Slider(
-                    minimum=0, maximum=2, value=1, step=0.01, label="Temperature"
-                ),
-                gr.Text(label="LoRA Name"),
-            ],
+                    if (
+                        "reasoning_content" in delta
+                        and delta["reasoning_content"] is not None
+                        and is_first_reasoning_content
+                    ):
+                        reasoning_content += html.escape(delta["reasoning_content"])
+                        history.append(
+                            ChatMessage(
+                                role="assistant",
+                                content=reasoning_content,
+                                metadata={"title": "💭 Thinking Process"},
+                            )
+                        )
+                        is_first_reasoning_content = False
+                    elif (
+                        "reasoning_content" in delta
+                        and delta["reasoning_content"] is not None
+                    ):
+                        reasoning_content += html.escape(delta["reasoning_content"])
+                        history[-1] = ChatMessage(
+                            role="assistant",
+                            content=reasoning_content,
+                            metadata={"title": "💭  Thinking Process"},
+                        )
+                    elif (
+                        "content" in delta
+                        and delta["content"] is not None
+                        and is_first_content
+                    ):
+                        response_content += html.escape(delta["content"])
+                        history.append(
+                            ChatMessage(role="assistant", content=response_content)
+                        )
+                        is_first_content = False
+                    elif "content" in delta and delta["content"] is not None:
+                        response_content += html.escape(delta["content"])
+                        # Replace thinking message with actual response
+                        history[-1] = ChatMessage(
+                            role="assistant", content=response_content
+                        )
+                    yield history
+            else:
+                result = model.chat(
+                    messages=messages,
+                    generate_config={
+                        "max_tokens": int(max_tokens),
+                        "temperature": temperature,
+                        "lora_name": lora_name,
+                    },
+                )
+                assert isinstance(result, dict)
+                mg = result["choices"][0]["message"]
+                if "reasoning_content" in mg:
+                    reasoning_content = mg["reasoning_content"]
+                    if reasoning_content is not None:
+                        reasoning_content = html.escape(str(reasoning_content))
+                        history.append(
+                            ChatMessage(
+                                role="assistant",
+                                content=reasoning_content,
+                                metadata={"title": "💭 Thinking Process"},
+                            )
+                        )
+
+                content = mg["content"]
+                response_content = (
+                    html.escape(str(content)) if content is not None else ""
+                )
+                history.append(ChatMessage(role="assistant", content=response_content))
+                yield history
+
+        with gr.Blocks(
             title=f"🚀 Xinference Chat Bot : {self.model_name} 🚀",
             css="""
             .center{
@@ -165,25 +215,123 @@ class GradioInterface:
                 padding: 0px;
                 color: #9ea4b0 !important;
             }
-            """,
-            description=f"""
-            <div class="center">
-            Model ID: {self.model_uid}
-            </div>
-            <div class="center">
-            Model Size: {self.model_size_in_billions} Billion Parameters
-            </div>
-            <div class="center">
-            Model Format: {self.model_format}
-            </div>
-            <div class="center">
-            Model Quantization: {self.quantization}
-            </div>
+            .main-container {
+                display: flex;
+                flex-direction: column;
+                padding: 0.5rem;
+                box-sizing: border-box;
+                gap: 0.25rem;
+                flex-grow: 1;
+                min-width: min(320px, 100%);
+                height: calc(100vh - 70px)!important;
+            }
+            .header {
+                flex-grow: 0!important;
+            }
+            .header h1 {
+                margin: 0.5rem 0;
+                font-size: 1.5rem;
+            }
+            .center {
+                font-size: 0.9rem;
+                margin: 0.1rem 0;
+            }
+            .chat-container {
+                flex: 1;
+                display: flex;
+                min-height: 0;
+                margin: 0.25rem 0;
+            }
+            .chat-container .block {
+                height: 100%!important;
+            }
+            .input-container {
+                flex-grow: 0!important;
+            }
             """,
             analytics_enabled=False,
-        )
+        ) as chat_interface:
+            with gr.Column(elem_classes="main-container"):
+                # Header section
+                with gr.Column(elem_classes="header"):
+                    gr.Markdown(
+                        f"""<h1 style='text-align: center; margin-bottom: 1rem'>🚀 Xinference Chat Bot : {self.model_name} 🚀</h1>"""
+                    )
+                    gr.Markdown(
+                        f"""
+                        <div class="center">Model ID: {self.model_uid}</div>
+                        <div class="center">Model Size: {self.model_size_in_billions} Billion Parameters</div>
+                        <div class="center">Model Format: {self.model_format}</div>
+                        <div class="center">Model Quantization: {self.quantization}</div>
+                        """
+                    )
 
-    def build_chat_vl_interface(
+                # Chat container
+                with gr.Column(elem_classes="chat-container"):
+                    chatbot = gr.Chatbot(
+                        type="messages",
+                        label=self.model_name,
+                        show_label=True,
+                        render_markdown=True,
+                        container=True,
+                    )
+
+                # Input container
+                with gr.Column(elem_classes="input-container"):
+                    with gr.Row():
+                        with gr.Column(scale=12):
+                            textbox = gr.Textbox(
+                                show_label=False,
+                                placeholder="Type a message...",
+                                container=False,
+                            )
+                        with gr.Column(scale=1, min_width=50):
+                            submit_btn = gr.Button("Enter", variant="primary")
+
+                    with gr.Accordion("Additional Inputs", open=False):
+                        max_tokens = gr.Slider(
+                            minimum=1,
+                            maximum=self.context_length,
+                            value=512
+                            if "reasoning" not in self.model_ability
+                            else self.context_length // 2,
+                            step=1,
+                            label="Max Tokens",
+                        )
+                        temperature = gr.Slider(
+                            minimum=0,
+                            maximum=2,
+                            value=1,
+                            step=0.01,
+                            label="Temperature",
+                        )
+                        stream = gr.Checkbox(label="Stream", value=True)
+                        lora_name = gr.Text(label="LoRA Name")
+
+            # deal with message submit
+            textbox.submit(
+                lambda m, h: ("", h + [ChatMessage(role="user", content=m)]),
+                [textbox, chatbot],
+                [textbox, chatbot],
+            ).then(
+                generate_wrapper,
+                [textbox, chatbot, max_tokens, temperature, lora_name, stream],
+                chatbot,
+            )
+
+            submit_btn.click(
+                lambda m, h: ("", h + [ChatMessage(role="user", content=m)]),
+                [textbox, chatbot],
+                [textbox, chatbot],
+            ).then(
+                generate_wrapper,
+                [textbox, chatbot, max_tokens, temperature, lora_name, stream],
+                chatbot,
+            )
+
+            return chat_interface
+
+    def build_chat_multimodel_interface(
         self,
     ) -> "gr.Blocks":
         def predict(history, bot, max_tokens, temperature, stream):
@@ -194,15 +342,10 @@ class GradioInterface:
             model = client.get_model(self.model_uid)
             assert isinstance(model, RESTfulChatModelHandle)
 
-            prompt = history[-1]
-            assert prompt["role"] == "user"
-            prompt = prompt["content"]
-            # multimodal chat does not support stream.
             if stream:
                 response_content = ""
                 for chunk in model.chat(
-                    prompt=prompt,
-                    chat_history=history[:-1],
+                    messages=history,
                     generate_config={
                         "max_tokens": max_tokens,
                         "temperature": temperature,
@@ -227,8 +370,7 @@ class GradioInterface:
                 yield history, bot
             else:
                 response = model.chat(
-                    prompt=prompt,
-                    chat_history=history[:-1],
+                    messages=history,
                     generate_config={
                         "max_tokens": max_tokens,
                         "temperature": temperature,
@@ -236,11 +378,46 @@ class GradioInterface:
                     },
                 )
                 history.append(response["choices"][0]["message"])
-                bot[-1][1] = history[-1]["content"]
-                yield history, bot
+                if "audio" in history[-1]:
+                    # audio output
+                    audio_bytes = base64.b64decode(history[-1]["audio"]["data"])
+                    audio_file = tempfile.NamedTemporaryFile(
+                        delete=False, suffix=".wav"
+                    )
+                    audio_file.write(audio_bytes)
+                    audio_file.close()
 
-        def add_text(history, bot, text, image):
-            logger.debug("Add text, text: %s, image: %s", text, image)
+                    def audio_to_base64(audio_path):
+                        with open(audio_path, "rb") as audio_file:
+                            return base64.b64encode(audio_file.read()).decode("utf-8")
+
+                    def generate_html_audio(audio_path):
+                        base64_audio = audio_to_base64(audio_path)
+                        audio_format = audio_path.split(".")[-1]
+                        return (
+                            f"<audio controls style='max-width:100%;'>"
+                            f"<source src='data:audio/{audio_format};base64,{base64_audio}' type='audio/{audio_format}'>"
+                            f"Your browser does not support the audio tag.</audio>"
+                        )
+
+                    bot[-1] = (bot[-1][0], history[-1]["content"])
+                    yield history, bot
+
+                    # append html audio tag instead of gr.Audio
+                    bot.append((None, generate_html_audio(audio_file.name)))
+                    yield history, bot
+                else:
+                    bot[-1][1] = history[-1]["content"]
+                    yield history, bot
+
+        def add_text(history, bot, text, image, video, audio):
+            logger.debug(
+                "Add text, text: %s, image: %s, video: %s, audio: %s",
+                text,
+                image,
+                video,
+                audio,
+            )
             if image:
                 buffered = BytesIO()
                 with PIL.Image.open(image) as img:
@@ -260,20 +437,85 @@ class GradioInterface:
                         },
                     ],
                 }
+            elif video:
+
+                def video_to_base64(video_path):
+                    with open(video_path, "rb") as video_file:
+                        encoded_string = base64.b64encode(video_file.read()).decode(
+                            "utf-8"
+                        )
+                    return encoded_string
+
+                def generate_html_video(video_path):
+                    base64_video = video_to_base64(video_path)
+                    video_format = video_path.split(".")[-1]
+                    html_code = f"""
+                    <video controls>
+                        <source src="data:video/{video_format};base64,{base64_video}" type="video/{video_format}">
+                        Your browser does not support the video tag.
+                    </video>
+                    """
+                    return html_code
+
+                display_content = f"{generate_html_video(video)}\n{text}"
+                message = {
+                    "role": "user",
+                    "content": [
+                        {"type": "text", "text": text},
+                        {
+                            "type": "video_url",
+                            "video_url": {"url": video},
+                        },
+                    ],
+                }
+
+            elif audio:
+
+                def audio_to_base64(audio_path):
+                    with open(audio_path, "rb") as audio_file:
+                        encoded_string = base64.b64encode(audio_file.read()).decode(
+                            "utf-8"
+                        )
+                    return encoded_string
+
+                def generate_html_audio(audio_path):
+                    base64_audio = audio_to_base64(audio_path)
+                    audio_format = audio_path.split(".")[-1]
+                    return (
+                        f"<audio controls style='max-width:100%;'>"
+                        f"<source src='data:audio/{audio_format};base64,{base64_audio}' type='audio/{audio_format}'>"
+                        f"Your browser does not support the audio tag.</audio>"
+                    )
+
+                display_content = f"{generate_html_audio(audio)}<br>{text}"
+                message = {
+                    "role": "user",
+                    "content": [
+                        {"type": "text", "text": text},
+                        {
+                            "type": "audio_url",
+                            "audio_url": {"url": audio},
+                        },
+                    ],
+                }
+
             else:
                 display_content = text
                 message = {"role": "user", "content": text}
             history = history + [message]
             bot = bot + [[display_content, None]]
-            return history, bot, "", None
+            return history, bot, "", None, None, None
 
         def clear_history():
             logger.debug("Clear history.")
-            return [], None, "", None
+            return [], None, "", None, None, None
 
         def update_button(text):
             return gr.update(interactive=bool(text))
 
+        has_vision = "vision" in self.model_ability
+        has_audio = "audio" in self.model_ability
+
         with gr.Blocks(
             title=f"🚀 Xinference Chat Bot : {self.model_name} 🚀",
             css="""
@@ -312,10 +554,29 @@ class GradioInterface:
             state = gr.State([])
             with gr.Row():
                 chatbot = gr.Chatbot(
-                    elem_id="chatbot", label=self.model_name, height=550, scale=7
+                    elem_id="chatbot", label=self.model_name, scale=7, min_height=900
                 )
                 with gr.Column(scale=3):
-                    imagebox = gr.Image(type="filepath")
+                    if has_vision:
+                        imagebox = gr.Image(type="filepath")
+                        videobox = gr.Video()
+                    else:
+                        imagebox = gr.Image(type="filepath", visible=False)
+                        videobox = gr.Video(visible=False)
+
+                    if has_audio:
+                        audiobox = gr.Audio(
+                            sources=["microphone", "upload"],
+                            type="filepath",
+                            visible=True,
+                        )
+                    else:
+                        audiobox = gr.Audio(
+                            sources=["microphone", "upload"],
+                            type="filepath",
+                            visible=False,
+                        )
+
                     textbox = gr.Textbox(
                         show_label=False,
                         placeholder="Enter text and press ENTER",
@@ -343,8 +604,8 @@ class GradioInterface:
 
             textbox.submit(
                 add_text,
-                [state, chatbot, textbox, imagebox],
-                [state, chatbot, textbox, imagebox],
+                [state, chatbot, textbox, imagebox, videobox, audiobox],
+                [state, chatbot, textbox, imagebox, videobox, audiobox],
                 queue=False,
             ).then(
                 predict,
@@ -354,8 +615,8 @@ class GradioInterface:
 
             submit_btn.click(
                 add_text,
-                [state, chatbot, textbox, imagebox],
-                [state, chatbot, textbox, imagebox],
+                [state, chatbot, textbox, imagebox, videobox, audiobox],
+                [state, chatbot, textbox, imagebox, videobox, audiobox],
                 queue=False,
             ).then(
                 predict,
@@ -364,7 +625,10 @@ class GradioInterface:
             )
 
             clear_btn.click(
-                clear_history, None, [state, chatbot, textbox, imagebox], queue=False
+                clear_history,
+                None,
+                [state, chatbot, textbox, imagebox, videobox, audiobox],
+                queue=False,
             )
 
         return chat_vl_interface
@@ -428,7 +692,7 @@ class GradioInterface:
                     }
 
             hist.append(response_content)
-            return {
+            return {  # type: ignore
                 textbox: response_content,
                 history: hist,
             }
@@ -467,7 +731,7 @@ class GradioInterface:
                     }
 
             hist.append(response_content)
-            return {
+            return {  # type: ignore
                 textbox: response_content,
                 history: hist,
             }
